/***************************************************************************
    kcdspeed.h  -  CD-ROM speed setting applet
    ------------------------------------------------------------------------
    begin                : Thu Jun  6 2002
    copyright            : (C) 2002, 2005 by Tomas Horsky
    email                : tomas@riadoklan.sk
 ***************************************************************************/

#ifndef KCDSPEED_H
#define KCDSPEED_H

#ifdef HAVE_CONFIG_H
#	include <config.h>
#endif

#include <kpanelapplet.h>
#include <qstring.h>

class KComboBox;
class KPushButton;

class KcdsSettings;
class KcdsPreferences;
class CdromDevice;

/**	Main applet widget. It has combobox for selecting speed, and two buttons
	to eject and close tray.
*/
class KCdSpeed: public KPanelApplet
{
	Q_OBJECT
	public:
		/**	Constructor
		*/
		KCdSpeed(const QString &configFile, Type t=Normal, int actions=0, QWidget *parent=0, const char *name=0);
		/**	Destructor
		*/
		virtual ~KCdSpeed();
		
		int widthForHeight(int height) const;
		int heightForWidth(int width) const;
		
	protected slots:
		void slotOpenDeviceError(const char *device);
		
		/**	Changes speed of CD-ROM device, possibly making changes in combobox.
		*/
		void slotSelectSpeed(const QString &speed);
		void slotSpeedSelected(int speed);
		void slotSpeedSelectError(int speed);
		
		/**	Ejects CD-ROM tray.
		*/
		void slotEjectTray();
		void slotTrayEjected();
		void slotTrayEjectError();
		/**	Closes CD-ROM tray.
		*/
		void slotCloseTray();
		void slotTrayClosed();
		void slotTrayCloseError();
		
		/**	Mounts CD.
		*/
		void slotMount();
		/**	Unmounts CD. If eject is true, ejects tray after unmount has finished.
		*/
		void slotUnmount(bool eject=false);
		
		/**	Opens filemanager with directory where CD is mounted.
		*/
		void slotOpenFilemanager();
		
		/**	Opens preferences dialog.
		*/
		void slotOpenPreferences();
		/**	Applies settings from preferences dialog.
		*/
		void slotPreferencesApply();
		/**	Called when preferences dialog gets destroyed.
		*/
		void slotPreferencesDeleted();
		
		/**	Shows dialog with information about device capabilities.
		*/
		void slotShowCapabilities();
		
		/**	Locks and unlocks E and C buttons.
		*/
		void slotToggleLockButtons();
		
	protected:
		virtual void about();
		virtual void preferences() {slotOpenPreferences();}
		
		/**	Sets widgets according to settings.
		*/
		void updateWidgets();
		/**	Inserts and shows (sets current item) string representing unknown speed "?".
			In future, when detection of current speed will work, this will be
			replaced by something like showCurrentSpeed().
		*/
		void showUnknownSpeed();
		/**	Removes string representing unknown speed "?", and makes sure that
			current item is not changed by this removal.
		*/
		void removeUnknownSpeed();
		/**	Looks for given speed in combobox and if it is there, shows it (sets current
			item), and returns true, else returns false.
		*/
		bool findSpeed(QString &speed);
		/**	Looks whether _speeds and _settings->availSpeeds() have the same
			speeds, or they differ.
		*/
		bool availSpeedsChanged();
		
		virtual void resizeEvent(QResizeEvent *event);
		virtual void mousePressEvent(QMouseEvent *event);
		virtual bool eventFilter(QObject *watched, QEvent *event);
		
	private:
		KcdsSettings *_settings;
		KcdsPreferences *_preferences;
		
		CdromDevice *_cdrom;
		
		KComboBox *_speeds;
		KPushButton *_eject;
		KPushButton *_close;
		
		int _prevItem;
};

#endif //KCDSPEED_H
